@@ -407,7 +407,7 @@ class Authentication():
             return self.authenticate_with_refresh(token_data)
         if self.access_token and not self.refresh_token:
             tokens = self.access_token.split('.')
-            inputdata = json.loads(base64.b64decode(tokens[1]))
+            inputdata = json.loads(base64.b64decode(tokens[1]+('='*(len(tokens[1])%4))))
             self.tokendata = {
                 'accessToken': self.access_token,
                 'tokenType': 'Bearer',
