@@ -2,9 +2,9 @@ import os
 import json
 import datetime
 import sqlalchemy.types
-from sqlalchemy import Column, Text, Boolean, BigInteger as Integer, Binary, create_engine, Table, ForeignKey
+from sqlalchemy import Column, Text, Boolean, BigInteger as Integer, create_engine, Table, ForeignKey
 from sqlalchemy.ext.declarative import declarative_base
-from sqlalchemy.orm import relationship, sessionmaker
+from sqlalchemy.orm import relationship, sessionmaker, foreign
 from sqlalchemy.types import TypeDecorator, TEXT
 Base = declarative_base()
 
@@ -121,6 +121,37 @@ lnk_role_member_serviceprincipal = Table('lnk_role_member_serviceprincipal', Bas
     Column('ServicePrincipal', Text, ForeignKey('ServicePrincipals.objectId'))
 )
 
+lnk_role_member_group = Table('lnk_role_member_group', Base.metadata,
+    Column('DirectoryRole', Text, ForeignKey('DirectoryRoles.objectId')),
+    Column('Group', Text, ForeignKey('Groups.objectId'))
+)
+
+class AppRoleAssignment(Base, SerializeMixin):
+    __tablename__ = "AppRoleAssignments"
+    objectType = Column(Text)
+    objectId = Column(Text, primary_key=True)
+    deletionTimestamp = Column(DateTime)
+    creationTimestamp = Column(DateTime)
+    id = Column(Text)
+    principalDisplayName = Column(Text)
+    principalId = Column(Text)
+    principalType = Column(Text)
+    resourceDisplayName = Column(Text)
+    resourceId = Column(Text)
+
+
+class OAuth2PermissionGrant(Base, SerializeMixin):
+    __tablename__ = "OAuth2PermissionGrants"
+    clientId = Column(Text)
+    consentType = Column(Text)
+    expiryTime = Column(DateTime)
+    objectId = Column(Text, primary_key=True)
+    principalId = Column(Text)
+    resourceId = Column(Text)
+    scope = Column(Text)
+    startTime = Column(DateTime)
+
+
 class User(Base, SerializeMixin):
     __tablename__ = "Users"
     objectType = Column(Text)
@@ -331,6 +362,13 @@ class ServicePrincipal(Base, SerializeMixin):
         back_populates="memberServicePrincipals")
 
 
+    oauth2PermissionGrants = relationship("OAuth2PermissionGrant",
+        primaryjoin=objectId == foreign(OAuth2PermissionGrant.clientId))
+
+    appRolesAssigned = relationship("AppRoleAssignment",
+        primaryjoin=objectId == foreign(AppRoleAssignment.principalId))
+
+
 class Group(Base, SerializeMixin):
     __tablename__ = "Groups"
     objectType = Column(Text)
@@ -403,6 +441,10 @@ class Group(Base, SerializeMixin):
         secondaryjoin=objectId==lnk_group_member_group.c.Group,
         back_populates="memberGroups")
 
+    memberOfRole = relationship("DirectoryRole",
+        secondary=lnk_role_member_group,
+        back_populates="memberGroups")
+
 
 class Application(Base, SerializeMixin):
     __tablename__ = "Applications"
@@ -534,6 +576,10 @@ class DirectoryRole(Base, SerializeMixin):
         secondary=lnk_role_member_serviceprincipal,
         back_populates="memberOfRole")
 
+    memberGroups = relationship("Group",
+        secondary=lnk_role_member_group,
+        back_populates="memberOfRole")
+
 
 class TenantDetail(Base, SerializeMixin):
     __tablename__ = "TenantDetails"
@@ -651,18 +697,6 @@ class Contact(Base, SerializeMixin):
         back_populates="memberContacts")
 
 
-class OAuth2PermissionGrant(Base, SerializeMixin):
-    __tablename__ = "OAuth2PermissionGrants"
-    clientId = Column(Text)
-    consentType = Column(Text)
-    expiryTime = Column(DateTime)
-    objectId = Column(Text, primary_key=True)
-    principalId = Column(Text)
-    resourceId = Column(Text)
-    scope = Column(Text)
-    startTime = Column(DateTime)
-
-
 class Policy(Base, SerializeMixin):
     __tablename__ = "Policys"
     objectType = Column(Text)
@@ -699,20 +733,6 @@ class RoleAssignment(Base, SerializeMixin):
     roleDefinitionId = Column(Text)
 
 
-class AppRoleAssignment(Base, SerializeMixin):
-    __tablename__ = "AppRoleAssignments"
-    objectType = Column(Text)
-    objectId = Column(Text, primary_key=True)
-    deletionTimestamp = Column(DateTime)
-    creationTimestamp = Column(DateTime)
-    id = Column(Text)
-    principalDisplayName = Column(Text)
-    principalId = Column(Text)
-    principalType = Column(Text)
-    resourceDisplayName = Column(Text)
-    resourceId = Column(Text)
-
-
 def parse_db_argument(dbarg):
     '''
     Parse DB string given as argument into full path required
